/***************************************************************************
 *cr
 *cr            (C) Copyright 1995-2007 The Board of Trustees of the
 *cr                        University of Illinois
 *cr                         All Rights Reserved
 *cr
 ***************************************************************************/
/***************************************************************************
 * RCS INFORMATION:
 *
 *      $RCSfile: vmd.frag,v $
 *      $Author: johns $        $Locker:  $             $State: Exp $
 *      $Revision: 1.49 $       $Date: 2007/03/03 18:10:58 $
 *
 ***************************************************************************
 * DESCRIPTION:
 *  This file contains the VMD OpenGL fragment shader implementing
 *  per-pixel lighting with phong highlights etc.
 ***************************************************************************/

// requires GLSL version 1.10
#version 110

//
// Fragment shader varying and uniform variable definitions for data
// supplied by VMD and/or the vertex shader
//
varying vec3 oglnormal;       // interpolated normal from the vertex shader
varying vec3 oglcolor;        // interpolated color from the vertex shader
varying vec3 V;               // view direction vector
uniform vec3 vmdlight0;       // VMD directional lights
uniform vec3 vmdlight1;
uniform vec3 vmdlight2;
uniform vec3 vmdlight3;

uniform vec3 vmdlight0H;      // Blinn halfway vectors for all four lights
uniform vec3 vmdlight1H;
uniform vec3 vmdlight2H;
uniform vec3 vmdlight3H;

uniform vec4 vmdlightscale;   // VMD light on/off state for all 4 VMD lights,
                              // represented as a scaling constant.  Could be
                              // done with on/off flags but ATI doesn't deal
                              // well with branching constructs, so this value
                              // is simply multiplied by the light's
                              // contribution.  Hacky, but it works for now.

uniform vec4 vmdmaterial;     // VMD material properties
                              // [0] is ambient (white ambient light only)
                              // [1] is diffuse
                              // [2] is specular
                              // [3] is shininess

uniform float vmdopacity;     // VMD global alpha value
uniform float vmdoutline;     // VMD outline shading
uniform float vmdoutlinewidth;// VMD outline shading width

uniform int vmdfogmode;       // VMD depth cueing / fog mode

uniform int vmdtexturemode;   // VMD texture mode 0=off 1=modulate 2=replace
uniform sampler3D vmdtex0;    // active 3-D texture map

//
// VMD Fragment Shader
//
void main(void) {

  // Flip the surface normal if it is facing away from the viewer,
  // determined by polygon winding order provided by OpenGL.
  vec3 N = normalize(oglnormal);
  if (!gl_FrontFacing) {
    N = -N;
  }

  // beginning of shading calculations
  float ambient = vmdmaterial[0];
  float diffuse = vmdmaterial[1];
  float specular = vmdmaterial[2];
  float shininess = vmdmaterial[3];

  // Attempt to do cel shading
  // TODO: do with a mix
  vec3 objcolor = oglcolor;
  const float A = 0.2;
  const float B = 0.4;
  const float C = 0.6;
  const float D = 1.0;
  float df = max(0.0, dot(N, vmdlight0));
  if (df < A) df = 0.0;
  else if (df < B) df = B;
  else if (df < C) df = C;
  else df = D;
  float sf = max(0.0, dot(N, normalize(vmdlight0 + vec3(0,0,1))));
  sf = pow(sf, shininess);
  sf = step(0.5, sf);
  objcolor += (ambient + df*diffuse + sf*specular);

  // Attempt to outline based on lights
  float costheta = abs(dot(normalize(V), N));
  if (costheta < mix(0.1, 0.5, max(0.0, abs(dot(N, vmdlight0))))) {
    objcolor = vec3(0., 0., 0.);
  }

  // Fog computations
  const float Log2E = 1.442695; // = log2(2.718281828)
  float fog = 1.0;
  if (vmdfogmode == 1) {
    // linear fog
    fog = (gl_Fog.end - gl_FogFragCoord) * gl_Fog.scale;
  } else if (vmdfogmode == 2) {
    // exponential fog
    fog = exp2(-gl_Fog.density * gl_FogFragCoord * Log2E);
  } else if (vmdfogmode == 3) {
    // exponential-squared fog
    fog = exp2(-gl_Fog.density * gl_Fog.density * gl_FogFragCoord * gl_FogFragCoord * Log2E);
  }
  fog = clamp(fog, 0.0, 1.0);       // clamp the final fog parameter [0->1)

  gl_FragColor = vec4(mix(vec3(gl_Fog.color), objcolor, fog), vmdopacity);
}


